/*
 * GridEdgeEncoder.cpp
 *
 *  Created on: Jul 20, 2009
 *      Author: kottlers
 */


#include "GridEdgeEncoder.hpp"
//#include "Algebra.hpp"


GridEdgeEncode::GridEdgeEncode(Graph& g,BasicSolver& s,Translator& translator)
:	BasicEncoder<GENode,GECell>(g,s,translator){};


GECell::GECell(uint r, uint c,vector<GENode*>& nodes, vector<GECell*>& others, bool may_be_empty, Translator& trans,
				bool first_vertex_allowed,bool sec_vertex_allowed)
	:BasicCell<GENode,GECell>(r,c){

	if(first_vertex_allowed)		{addNode(nodes[0],trans);}
	if(sec_vertex_allowed)			{addNode(nodes[1],trans);}
	for(uint i=2;i<nodes.size();++i){addNode(nodes[i],trans);}

	char buf[50];
	for(uint i=0;i!= others.size();++i){
		GECell* other = others[i];
		sprintf(buf,"Cell [%d,%d] is connected to cell [%d,%d] ",other->row, other->col,row,col);
		Var* edge_var 		= trans.newVar(buf);
		other->edges		.push_back(this);
		other->edges_vars	.push_back(edge_var);
		this->edges			.push_back(other);
		this->edges_vars	.push_back(edge_var);
	}


	if(may_be_empty){
		sprintf(buf,"! At cell [%d,%d] is no vertex ",row,col);
		Var* empty 			= trans.newVar(buf);
		this->addNodeMayBeEmptyConstraint(empty);

	}
}

/*
Var* GECell::getConditionNoNodeAtThis(){
	 if(possible_nodes.empty()) return NULL;
	 return possible_nodes.back() == NULL ? possible_nodes_vars.back() : NULL;
}*/






bool GridEdgeEncode::initGrid(uint w, uint h){

	if(!initGridSize(w,h)) return false;

	for(uint i = 1; i<= vertex_count;++i){nodes.push_back(new GENode(Vertex(i)));}

	bool allow_empty_cells = width * height > vertex_count;


	assert(cells.empty());
	uint upper 	= (height+1)/2;
	uint left 	= (width+1)/2;
	const bool not_diagonal = height != width;

	for(uint row = 0; row != height; ++row){
		for(uint col=0;col != width; ++col){
			bool fst_node_allowed = row <= upper && col<= left;
			bool sec_node_allowed = not_diagonal || col >= row;
			GECell* cell = new GECell(row,col,nodes,cells,allow_empty_cells,trans,fst_node_allowed,sec_node_allowed);
			cells.push_back(cell);
		}
	}

	return true;
}


bool GridEdgeEncode::encode(){

	for(uint i=0;i!=nodes.size();++i){
		//card constraint
		if(!addCardConstraint(nodes[i]->getCellVars())) return false;
	}

	for(uint i=0;i!=cells.size();++i){
		GECell* cell1 = cells[i];
//		vector<Var*>& node_vars = cell1->getNodeVars();
		if(!addCardConstraint(cell1->getCardConstraintVars())) return false;

		vector<Var*> edges12 = cell1->getEdgeVars();
		for(uint k=0;k!= edges12.size();++k){
			GECell* cell2 = cell1->getEdge(k);
			assert(cell1 != cell2);
			Var* edge12 = edges12[k];

			//add a@1 && b@2 implies certain edge
			for(uint j=0;j!=cell1->nodesCount();++j){
				GENode* 	node1 			= cell1->getNode(j); assert(node1 != NULL);
				Var* 	node1_at_cell1 		= cell1->getVar(j);
				assert(node1_at_cell1 != NULL);

				vector<Vertex>& node_neighbours = graph.getAdjaOfVertex(node1->getVertex());

				for(uint m=0;m!= node_neighbours.size();++m){
					GENode* neighbour 		= vertex2Node(node_neighbours[m]);
					Var*  node2_at_cell2 	= neighbour->getConditionAtCell(cell2);
					if(node2_at_cell2 == NULL) continue;

					if(!addABImpliesC(node1_at_cell1,node2_at_cell2,edge12)) return false;
				}
			}

			//forbid crossing edges
			if(cellID(cell1) >= cellID(cell2))continue;
			for(uint j=0;j!=cells.size();++j){
				GECell* cell3 			= cells[j];
				if(cell3 == cell1 || cell3 ==cell2) continue;

				if(touches(cell1,cell2,cell3)){
					Var* cell3_free = cell3->getConditionNoNodeAtThis();
					if(!addAImpliesB(edge12,cell3_free))return false;
					if(cell3_free == NULL){	continue;}
				}

				vector<Var*> edges34 	= cell3->getEdgeVars();
				for(uint m=0;m!= edges34.size();++m){
					GECell* cell4 = cell3->getEdge(m);
					assert(cell4 != cell3);
					if(cell4 == cell1 || cell4 ==cell2 || cellID(cell3) >= cellID(cell4))continue;
					Var* edge34 = edges34[m];

					if(crosses(cell1,cell2,cell3,cell4)){
						if(!addNotAOrNotB(edge12,edge34)) return false;
					}
				}
			}

		}
	}


	return true;
}

#include "Algebra.hpp"

#define ec2P(c) Point(c->getRow(),c->getColumn())

bool GridEdgeEncode::crosses(GECell* c1,GECell* c2, GECell* d1, GECell* d2){
	return twoLinesCross(ec2P(c1),ec2P(c2),ec2P(d1),ec2P(d2));
}

bool GridEdgeEncode::touches(GECell* c1,GECell* c2, GECell* d){
	return contains(ec2P(c1),ec2P(c2),ec2P(d));
}




