/*
 * Grid.hpp
 *
 *  Created on: Jul 16, 2009
 *      Author: kottlers
 */

#ifndef GRID_HPP_
#define GRID_HPP_

#include <string>
using std::string;
#include "Graph.hpp"
#include <cmath>
#include "BasicEncoding.hpp"
#include "BasicSolver.hpp"


class GECell;

class GENode: public BasicNode<GECell>{
public:
	GENode(Vertex v):BasicNode<GECell>(v){};
};


class GECell: public BasicCell<GENode,GECell>{

	vector<GECell*>	edges;
	vector<Var*>	edges_vars;

	void addNode(GENode* node, Translator& trans){
		char buf[50];
		sprintf(buf,"! At cell [%d,%d] is vertex %d ",row,col,node->getID());
		Var* node_at_cell 	= trans.newVar(buf);
		addPossibleNode(node,node_at_cell);
		node->addPossibleCell(this,node_at_cell);
	}

public:
	GECell(uint r, uint c, vector<GENode*>& nodes, vector<GECell*>& others, bool may_be_empty, Translator& trans,
			bool first_node_allowed, bool second_node_allowed);

	GECell* 			getEdge(uint i){assert(i < edges.size()); return edges[i];}
	vector<Var*>&	getEdgeVars(){return edges_vars;}
};




class GridEdgeEncode: public BasicEncoder<GENode, GECell>{


	vector<GECell*>  cells;
	vector<GENode*>  nodes;

	bool crosses(GECell* c1,GECell* c2, GECell* d1, GECell* d2);
	bool touches(GECell* c1,GECell* c2, GECell* d);



	GENode* vertex2Node(const Vertex& v){
		assert(v.id <=nodes.size()&&v.id>0);
		assert(nodes[v.id-1]->getVertex().id == v.id);
		return nodes[v.id-1];}

public:
	GridEdgeEncode(Graph& g,BasicSolver& s,Translator& translator);
	~GridEdgeEncode(){
		for(uint i=cells.size();i--;){delete cells[i];}
		for(uint i=nodes.size();i--;){delete nodes[i];}
	}
	bool initGrid(uint width, uint height);
	bool encode();

};





#endif /* GRID_HPP_ */
